import { assertIsArray, normalizeFileName } from './utils';
import { downloadFile } from './processors';
import { _prepareData, _createJSONData, createCSVData, createXLSData, createXMLData } from './converters';
function exportFromJSON({ data, fileName = 'download', exportType = 'txt', replacer = null, space = 4, processor = downloadFile, withBOM = false, delimiter = ',', }) {
    const MESSAGE_IS_ARRAY_FAIL = 'Invalid export data. Please provide an array of object';
    const MESSAGE_UNKNOWN_EXPORT_TYPE = `Can't export unknown data type ${exportType}.`;
    const safeData = _prepareData(data);
    const JSONData = _createJSONData(safeData, replacer, space);
    switch (exportType) {
        case 'txt': {
            return processor(JSONData, exportType, normalizeFileName(fileName, 'txt'));
        }
        case 'json': {
            return processor(JSONData, exportType, normalizeFileName(fileName, 'json'));
        }
        case 'csv': {
            assertIsArray(safeData, MESSAGE_IS_ARRAY_FAIL);
            const BOM = '\ufeff';
            const CSVData = createCSVData(safeData, delimiter);
            const content = withBOM ? BOM + CSVData : CSVData;
            return processor(content, exportType, normalizeFileName(fileName, 'csv'));
        }
        case 'xls': {
            assertIsArray(safeData, MESSAGE_IS_ARRAY_FAIL);
            const content = createXLSData(safeData);
            return processor(content, exportType, normalizeFileName(fileName, 'xls'));
        }
        case 'xml': {
            const content = createXMLData(safeData);
            return processor(content, exportType, normalizeFileName(fileName, 'xml'));
        }
        default:
            throw new Error(MESSAGE_UNKNOWN_EXPORT_TYPE);
    }
}
(function (exportFromJSON) {
    exportFromJSON.types = {
        txt: 'txt',
        json: 'json',
        csv: 'csv',
        xls: 'xls',
        xml: 'xml',
    };
    exportFromJSON.processors = { downloadFile };
})(exportFromJSON || (exportFromJSON = {}));
export default exportFromJSON;
